#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# createBackup.pl  --method=<full|incr>  --img_type=<ftp|nfs> --pathlist=<colon seperated path list> [--svr_name=<server name>] [--img_name=<Backup Image Name>] [--img_dir=<Image Dir>] [--user_name=<User Name>] [--pasword=<Password>] [--help]
#
# Perform a backup of a given list of directories using either NFS or FTP.
#
#######################################################################

use strict;
use ASM::Common;
use ASM::Exec;
use ASM::Log;

my (@backupData_opts, $backupData_usage, $eSP_BIN, $PIDFILE, $BK_LIST_FN, $BK_STAT_FN);
my ($rc, $stdout, $stderr, $pathlist, $path, $script_name, $PID, $pid, $tformat, $ctime, $tarfn);
my (@cmdopts, $cmdopts, $stamp, @line, $bkupfile, $LOG);

@backupData_opts = ("status", "backup", "restore", "type=s", "protocol=s", "svr_name=s", "export_dir=s", "archive=s", "userid=s", "password=s");
$backupData_usage = "backupData --status|--backup|--restore --type=full|incremental --protocol=nfs|ftp --svr_name=server --export_dir=dir --archive=archive_directory --userid=user --password=password\n";

commonInit(\@backupData_opts, $backupData_usage);

use vars qw/ $BK_FS $BK_DIR $opt_status $opt_backup $opt_protocol $opt_svr_name $opt_archive $opt_type $opt_userid $opt_password $opt_export_dir $opt_restore /;

$eSP_BIN = "/opt/CSM/scripts";
$BK_FS = "/backup";
$BK_DIR = "/backup/installed";  #Formerly $bkup_home 
$PIDFILE = "/backup/backup.pid";
$BK_LIST_FN = "$BK_FS/backup.lst";
$BK_STAT_FN = "backup.stat";

$pathlist = "/home:/home2";

setFacility('backdata');

$rc = 0;

# Usage: 
#   cleanup_exit([MSG_CODE, arg1, arg2, ...], $exitrc);
sub cleanup_exit
{
  my ($msgargs, $exitrc) = @_;

   if (-f $PIDFILE)
   {
       my $PID = `cat $PIDFILE`;
       if ($$ == $PID)
       {
	 $rc = runit([ "rm", "-f", $PIDFILE], \$stdout, \$stderr);
       }
   }

  if ($exitrc != 77) {
    error(@$msgargs);
  }
   trace("Exiting: rc $exitrc");
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

sub sig_child
{
    my $pid = wait;

    $rc = $? >> 8;
    trace("process $$ caught child process with rc ($rc)\n");
}

&setup_signals(); 


if ($opt_status)
{
   if (alreadyRunning($PIDFILE, "backupData.pl"))
   {
      print getFormattedMessage(MSG_RUNNINGSTATUS) . "\n";
      exit(1);
   }
   else
   {
      print getFormattedMessage(MSG_NOTRUNNINGSTATUS) . "\n";
      exit(2);
   }
}
elsif ($opt_backup || $opt_restore)
{
    $LOG = "$BK_FS/backup.log";
    open LOG, "> $LOG";
    close STDERR;
    open STDERR, ">>&LOG";
    close STDOUT;
    open STDOUT, ">>&LOG";

    if (alreadyRunning($PIDFILE, "backupData.pl"))
    {
      cleanup_exit([MSG_BACKUP_RESTORE_IN_PROGRESS], 80);
    }

    # 
    # Check parms
    #

    if (!$opt_protocol)
    {
      cleanup_exit([MSG_OPTION_MISSING, "--protocol"], 2);
    }
    if (!$opt_svr_name)
    {
      cleanup_exit([MSG_OPTION_MISSING, "--svr_name"], 2);
    }
    if (!$opt_archive)
    {
      cleanup_exit([MSG_OPTION_MISSING, "--archive"], 2);
    }

    if ($opt_backup && !($opt_type eq "full" || $opt_type eq "incremental"))
    {
      cleanup_exit([MSG_OPTION_MISSING, "--full | --incremental"], 2);
    }

    if ($opt_protocol && !($opt_protocol eq "nfs" || $opt_protocol eq "ftp"))
    {
      cleanup_exit([MSG_INVALID_BACKUP_PROTOCOL], 2);
    }
    
    if ($opt_protocol eq "ftp" && (!$opt_userid))
    {
      cleanup_exit([MSG_E21], 2); # no userid
    }

    if ($opt_protocol eq "ftp" && (!$opt_password))
    {
      cleanup_exit([MSG_E22], 2); # no password
    }

    if ($opt_protocol eq "nfs" && !$opt_export_dir)
    {
      cleanup_exit([MSG_E70], 2); # remote export directory required
    }

    &spawn;
    #
    # Parms validated
    #
}
else
{
  usage_error();
}

cleanup_exit([MSG_BACKUP_RESTORE_COMPLETE], $rc);

sub usage_error
{
  cleanup_exit([MSG_USAGE_ERROR, $script_name, "$script_name [--backup|--restore|--status"], 2);
}


#
# Function: spawn
#
# Spawns (forks) the back or restore processes
#
sub spawn
{

  FORK:
    {
	$SIG{'CHLD'} = 'sig_child';

	$rc = 77;
	if ($pid = fork)
	{
	    trace "$pid spawned.\n";
	    sleep 10;
	    trace "$$ parent process done sleeping for 10 seconds.\n";
	    trace "Current rc, $rc.\n";
	    
	    $SIG{'CHLD'} = 'IGNORE';

            # Okay, if the child finished, $rc has changed to its
            # exit code.  It's already cleaned up, logged everything,
            # so we want to exit silently.
            exit($rc);
	}
	elsif (defined $pid)
	{
	    $SIG{'CHLD'} = 'IGNORE';

	    `echo $$ > $PIDFILE`;
	    
	    if ($opt_backup)
	    {
		&backup;
	    }
	    elsif ($opt_restore)
	    {
		&restore;
	    }
	    else
	    {
	      usage_error();
	    }
	}
	elsif ($! =~ /No more process/)
	{
	    sleep 5;
	    redo FORK;
	}
	else
	{
	  trace("fork failed: $!");
	  cleanup_exit([MSG_CANNOT_START_NEW_PROCESS], 2);
	}
    }
}


#
# Function: backup
#
# Performs the incremental/full backup
#
sub backup
{
  audit(MSG_E77);

    # Temp format of time since CMVC will recognize certain %x% combinatoins (SCCS)
    $tformat = "+%C"."%y"."-%m"."-%d"." %H".":%M".":%S";
    $ctime = `date \'$tformat\'`;
    chomp($ctime);
    $tarfn = $ctime;
    $tarfn =~ s/[:\s-]//g;

    @cmdopts = ("--img_type=$opt_protocol", "--localdir=$BK_FS", "--svr_name=$opt_svr_name");
    if ($opt_protocol eq "nfs")
    {
	push(@cmdopts, ("--svr_dir=$opt_export_dir", "--remotedir=$opt_archive"));
    }
    else #ftp
    {
	push(@cmdopts, ("--remotedir=$opt_archive","--user_name=$opt_userid", "--password=$opt_password"));
    }

    
    # List of directories/files to backup
    open(LST, "> $BK_LIST_FN") or &cleanup_exit("Unable to create $BK_LIST_FN\n", 67);
    foreach $path (split(/:/, $pathlist))
    {
	print LST "$path\n";
    }
    close LST;


    if ($opt_type eq "incremental")
    {
	#  Incremental Backup
	#  Get timestamp of last backup
	$rc = runit( ["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=get", "--filename=$BK_STAT_FN")], \$stdout, \$stderr);
	if ($rc != 0) 
	{
	  trace("couldn't get $BK_STAT_FN: rc=$rc\n$stdout\n$stderr");
	  cleanup_exit([MSG_E79], $rc);	# unable to determine time of last backup
	}
	    
	open (LST, "< $BK_FS/$BK_STAT_FN") or do {
	  trace("failed to open $BK_FS/$BK_STAT_FN: $!");
	  cleanup_exit([MSG_E79], $rc);	# unable to determine time of last backup
	  #&cleanup_exit("Error:  couldn't determine date of last backup.", 78);
	};
	$stamp = "";
	while (<LST>)
	{
	    chomp($_);
	    @line = split(/=/, $_);
	    if ($line[0] eq "date")
	    {
		$stamp = $line[1];
	    }
	}
	close LST;
	if ($stamp eq "")
	{
	  trace("did not find date in file");
	  cleanup_exit([MSG_E79], $rc);	# unable to determine time of last backup
	}

	#  Perform backup
	$rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=put", "--tarcompress=$tarfn", "--filelist=$BK_LIST_FN", "--tarafter=$stamp")], \$stdout, \$stderr);
	if ($rc != 0) { trace("$stdout\n$stderr"); }
    }
    else
    {
	trace "Removing old backup sets.\n";
	$rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=remove", "--filename=*")], \$stdout, \$stderr);
	if ($rc != 0)
	{
	    trace "Error:  couldn't cleanup remote backup directory. Continuing anyway....\n";
	}

	#  Perform backup
	$rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=put", "--tarcompress=$tarfn", "--filelist=$BK_LIST_FN")], \$stdout, \$stderr);
	if ($rc != 0) { trace("$stdout\n$stderr"); }
    }
    
    if ($rc == 0)
    {
	if ($opt_type eq "full")
        {
	    open (STATS, "> $BK_FS/$BK_STAT_FN") or do {
	      trace("error creating file $BK_FS/$BK_STAT_FN: $!");
	      cleanup_exit([MSG_COULD_NOT_CREATE_FILE, "$BK_FS/$BK_STAT_FN"],
			   67);
	    };
	}
	else
	{
	    open (STATS, ">> $BK_FS/$BK_STAT_FN") or do {
	      trace("error appending to file $BK_FS/$BK_STAT_FN: $!");
	      cleanup_exit([MSG_COULD_NOT_CREATE_FILE, "$BK_FS/$BK_STAT_FN"],
			   67);
	    };
	}

	print STATS "date=$ctime\n";
	print STATS "$tarfn.tar.gz\n";
	close STATS;
	
	$rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=put", "--filename=$BK_STAT_FN")], \$stdout, \$stderr);
    }
    
  cleanup_exit([MSG_BACKUP_RESTORE_COMPLETE], $rc);
}

#
# Function: restore
#
# Restore backup data
#
sub restore
{
    audit(MSG_E77);		# backup/restore started

    @cmdopts = ("--img_type=$opt_protocol", "--svr_name=$opt_svr_name", "--remotedir=$opt_archive");
    if ($opt_protocol eq "nfs")
    {
	push(@cmdopts, "--svr_dir=$opt_export_dir");
    }
    else # FTP restore
    {
	push(@cmdopts, "--remotedir=$opt_archive", "--user_name=$opt_userid", "--password=$opt_password");
    }

    trace "Retrieving $BK_STAT_FN\n";
    $rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=get", "--localdir=$BK_FS", "--filename=$BK_STAT_FN")], \$stdout, \$stderr);
    if ($rc != 0) 
    {
      trace("failed: rc=$rc\n$stdout\n$stderr");
      cleanup_exit([MSG_E79], 78);
    }

    # Which Home drives are mounted?
    my $tarfilter = "";
    my @fsfiles;
    open FSTAB, "< /proc/mounts" or do {
	trace("error opening /proc/mounts: $!");
	cleanup_exit([MSG_COULD_NOT_OPEN_FILE, "/proc/mounts"], 67);
    };
    while (<FSTAB>)
    {
	next if (/^\#/ || /^\s*$/);
	my ($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	next if !defined($fs_spec);
	if ($fs_file eq "/home" ||
	    $fs_file eq "/home2")
	{
	  push(@fsfiles, $fs_file);
	}   
    }
    close FSTAB;
    $tarfilter = join ' ', @fsfiles;

    open (LST, "< $BK_FS/$BK_STAT_FN") or do {
	trace("error opening $BK_FS/$BK_STAT_FN: $!");
	cleanup_exit([MSG_COULD_NOT_OPEN_FILE, "$BK_FS/$BK_STAT_FN"], 67);
      };
    while (<LST>)
    {
	chomp($_);
	$bkupfile = $_;
	@line = split(/=/, $bkupfile);
	if ($line[0] ne "date")
	{
	    trace("Performing a restore using backup file: $bkupfile\n");
	    $rc = runit(["$eSP_BIN/xferFiles.pl", (@cmdopts, "--xfer=get", "--localdir=/", "--tarcompress=$tarfilter", "--filename=$bkupfile")], \$stdout, \$stderr);
	    if ($rc)
	    {
	      trace("failed: rc=$rc\n$stdout\n$stderr");
	      cleanup_exit([MSG_RESTORE_FAILED], $rc);
	    }
	}
    }
    close LST;
    
  cleanup_exit([MSG_BACKUP_RESTORE_COMPLETE], 0);
}


