#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# device.pl - Sets and gets the network info for the network interface devices.
#
# Paul Schroeder <paulsch@us.ibm.com> (30 September 1999)
#
#

use ASM::Common;
use ASM::Exec;

$device_usage = <<"EOS";
device.pl --get [ DEVICE, --device=DEVICE ] [ --all ] [--current]
device.pl --del, --delete  --device=DEVICE, DEVICE
device.pl --set DEVICE, --device=DEVICE  [ --ipaddr=IPADDR ] [ --netmask=NETMASK ]  [ --onboot=[yes|no] ] [ --bootproto=PROTO ]
EOS

commonInit(["get", "set", "all", "delete", "device=s", "ipaddr=s", "netmask=s", "onboot=s", "bootproto=s", "current"],
	   $device_usage);

setFacility('netcfg');

use Socket;
use DB_File;

$opt_device = shift(@ARGV) if $ARGV[0];

my @network_parms = ('DEVICE', 'IPADDR', 'NETMASK', 'NETWORK', 'BROADCAST', 'ONBOOT', 'BOOTPROTO', 'HWADDR');


if($opt_get) {
    if($opt_all) {
	# We want a listing of all the devices here so...
	# Just list the device files with ls and pipe 'em in...
        # open(DEVICES, "ls $device_file* |") || die "Can't iterate over devices: $!";
      my @devices = glob("$device_file*");
	# call getDevice for each device.
      foreach (@devices) {
	    next if /~$/;	# skip Emacs backup files
	    s/$device_file//;
	    %device_values = &getDevice($_);
	    print join($sep, @device_values{@network_parms}) . "\n";
	}#while
    } elsif($opt_device) {
	%device_values = &getDevice($opt_device);
	$device_values = join($sep, @device_values{@network_parms}) . "\n";
	print $device_values if $device_values{"DEVICE"} ne "";
    } else {
      logUsage();
      exit(3);
    }#if-elsif-else
    } elsif($opt_delete) {
	if (!$opt_device) {
	  error(LOG_DEVICE5);	# no device specified
	  exit(5);
	}
	&deleteDevice($opt_device);
    } elsif($opt_set) {
	if (!$opt_device) {
	  error(LOG_DEVICE5);	# no device specified
	  exit(5);
	}
	if ($opt_ipaddr && !&goodIP($opt_ipaddr)) {
	  error(LOG_HOSTS5);	# invalid IP address
	  exit(6);
	}
	if ($opt_netmask && !&goodIP($opt_netmask)) {
	  error(MSG_DEVICE7);	# invalid netmask
	  exit(7);
	}
	if ($opt_onboot && $opt_onboot !~ /^(yes|no)$/i) {
	  logUsage();
	}
	&setDevice($opt_device, $opt_ipaddr, $opt_netmask, $opt_network, $opt_broadcast, $opt_onboot, $opt_bootproto);
    } else {
      logUsage();
    }#if-elsif-elsif-else
    
    exit(0);


################################
# Lookout!  Subroutines below! #
################################

sub getDevice {
	my $device = shift(@_);
	my %network = ();

	# Get the HW Address if available
	$network{HWADDR} = '?';
	if ($device !~ /^lo/) {	# loopback has no hardware
	  local(*DEVICE);
	  if (open(DEVICE, "/sbin/ifconfig $device 2>/dev/null |")) {
	    my ($line) = <DEVICE>;
	    chomp($line);
	    if ($line =~ /HWaddr\s+([0-9A-F:]+)/i) {
	      $network{HWADDR} = $1;
	    } else {
	      trace("looking for HWAddr, strange line from ifconfig: $line\n");
	    }
	    close(DEVICE);
	  }
	}

	if ($opt_current) {
	  # Get the current setting, not the configured setting
	  local(*DEVICE);
	  if (open(DEVICE, "/sbin/ifconfig $device 2>/dev/null |")) {
	    $network{DEVICE} = $device;
	    while (<DEVICE>) {
	      if (/inet addr:([0-9\.]+)/i) {
		$network{IPADDR} = $1;
	      }
	      if (/Mask:([0-9\.]+)/i) {
		$network{NETMASK} = $1;
	      }
	      if (/Bcast:([0-9\.]+)/i) {
		$network{BROADCAST} = $1;
	      }
	      if (/HWaddr\s+([0-9A-F:]+)/i) {
		$network{HWADDR} = $1;
	      }
	    }
	    close(DEVICE);
	    return %network;
	  }
	}

	# If the device file doesn't exist just return the empty network scalar.
	return %network if ! -e "$device_file$device";

	openOrDie(DEVICE, "<$device_file$device");

	# Read each line getting the key-value pair.
	# $. is the perl special var which equals the last line number read from
	# the last file handle read.
	while(<DEVICE>) {
		chop;  # remove newline from end
                s/#.*//;   # remove comments
		($key, $value) = split(/=/);
		$network{uc($key)} = $value;
	}#while
	close(DEVICE);

	return %network;
}#getDevice

# Writes the device file with new options specified.
sub setDevice {
	my @new_network = @_;
	my $device = $new_network[0];
	my %network = &getDevice($device);
	my @network_parms = ('DEVICE', 'IPADDR', 'NETMASK', 'NETWORK', 'BROADCAST', 'ONBOOT', 'BOOTPROTO');

	$new_network[1] = $network{'IPADDR'} if !$new_network[1];
	$new_network[2] = $network{'NETMASK'} if !$new_network[2];
	$new_network[3] = inet_ntoa(inet_aton($new_network[1]) & inet_aton($new_network[2]));
	$new_network[4] = inet_ntoa(inet_aton($new_network[1]) & inet_aton($new_network[2]) | ~(inet_aton($new_network[2])) );

#       place our new hostname into /etc/hosts
	if ($device eq "eth0") {
	    modifyEtcHosts($network{'IPADDR'},$new_network[1]);
	}#endif

	$ifconfig = "/sbin/ifconfig $device $new_network[1] netmask $new_network[2] broadcast $new_network[4]";
	$rc = runit([$ifconfig], \$stdout, \$stderr);
	if ($rc) {
	  trace("ifconfig failed: rc=$rc\n$stdout\n$stderr");
	  error(MSG_DEVICE9);	# could not configure device
	  exit(9);
	}

	$new_network[5] = "yes" if !$network{'ONBOOT'} && !$new_network[5];
	$new_network[6] = "none" if !$network{'BOOTPROTO'} && !$new_network[6];

	openOrDie(DEVICE, ">$device_file$device");
	# $#scalar_array equals the number of elements in the array.
	$parms = "";
	foreach $i ( 0 .. $#network_parms) {
		# Assign the old value to new_network unless a new value
		# is specified.
		$new_network[$i] = $network{$network_parms[$i]} unless $new_network[$i];
		print DEVICE $network_parms[$i] . "=" . $new_network[$i] . "\n";
		if ($i !=0)
		{
		  $parms = "$parms $network_parms[$i]=$new_network[$i]";
		}
	}#for
	close(DEVICE);
	audit(MSG_UPDATED_DEVICE_CONFIGURATION, $device, $parms);
}#setDevice

# Removes the device file
sub deleteDevice {
	my $device = shift(@_);

	$rc = runit(["/sbin/ifconfig $device down"], \$stdout, \$stderr);
	if ($rc) {
	  trace("failed to configure device $device down: rc=$rc\n$stdout\n$stderr");
	  # Continue anyway, they wanted it deleted
	  #exit(10);
	}

	if (!unlink("$device_file$device")) {
	  trace("error deleting $device_file$device: $!");
	  error(MSG_COULD_NOT_DELETE_FILE, "$device_file$device");
	  exit(11);
	}
	audit(MSG_DEVICE_DELETED, $device);

}#deleteDevice
sub modifyEtcHosts{
	my $old_ipaddress = shift(@_);
	my $new_ipaddress = shift(@_);
	my $hostname;
	# open /etc/hosts for read/write to edit hostname entry
	tie (@file, 'DB_File', "$hosts_file", O_RDWR, 0644, $DB_RECNO) or print "Can't update $hosts_file: $!\n" and exit(5);
	my $i = 0;
        my $found = 0;

	while (defined $file[$i]) {

        	$_ = $file[$i];

		#find the current hostname line
		if (/$old_ipaddress\s+/){
		    $found = 1;
	            last;
        	}
        	$i++;

	} #while

	
	if ($found) {
	    $file[$i] =~ s/$old_ipaddress/$new_ipaddress/;
	    untie @file;
	} else {  # no hostname entry found just append it to the end with ipaddress of eth0
	    untie @file;
	    openOrDie(HOSTNAME, "<$hostname_file");
	    while($hostname = <HOSTNAME>) {
		chop $hostname;  # remove newline from end
		last;
	    }#while
	    close(HOSTNAME);
	    if ( $hostname =~ /\w+/) {
	      openOrDie(HOSTS, ">>$hosts_file");
	      print HOSTS $new_ipaddress. "\t" . $hostname . "\n";
	      close(HOSTS);
	    }
	}
}












