#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2001. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# nfs.pl implements nfs filesystem functions for ASM.
# Bernd Horn <bhorn@us.ibm.com> (April 23, 2001)

use ASM::Exec;
use ASM::Common;
use File::Copy;
use File::Path;

commonInit( [ "list:s",
	      "add",
	      "delete=s",
	      "edit=s",
	      "mount:s",
	      "unmount:s",
	      "hostname=s",
	      "export=s",
	      "options=s",
	      "debug" ],
	    "--list [mount_point]|--add|--delete|--edit");

use vars qw! $opt_list $opt_add $opt_delete $opt_mount $opt_unmount $opt_hostname 
             $opt_export $opt_options $opt_debug !;

#
#
#
#

sub printNFSFilesystems();
sub getFstab();
sub getProcMounts();
sub getFilesystemTable($);

##############################################################################
#
# Check parameters
#
#if (! ($opt_list || $opt_add || $opt_delete || $opt_activate || $opt_deactivate)) {
#  logUsage();
#  exit(1);
#}

##############################################################################
#
# Define variables
#
my $DEBUG = 0;
$DEBUG = 1 if $opt_debug;

# Filesystem Table
my $fstab_file = "/etc/fstab";


##############################################################################
#
# 
#
if (defined($opt_list))
{
    printNFSFilesystems();
}
elsif (defined($opt_add))
{
    goodHostnameOrIPOrDie($opt_hostname);
    goodNFSExportOrDie($opt_export); 
    goodNFSOptionsOrDie($opt_options);

    if (!defined($opt_options) ||
	$opt_options eq "")
    {
	$opt_options = "defaults";
    }
	
    my $mount_point = addNFSFilesystem();
    print "mount point $mount_point added\n";
    if (defined($opt_mount))
    {
	print "mounting $mount_point\n";
	exit(2) if (!mountFilesystem($mount_point));
    }
    exit(0);
}
elsif (defined($opt_delete))
{
    if ($opt_delete ne "")
    {
	my %hash = getProcMounts();
	
	if ($hash{$opt_delete})
	{
	    exit(3) if (!unmountFilesystem($opt_delete));
	}      
	deleteFilesystem($opt_delete);
    }
}
elsif (defined($opt_edit))
{
    goodHostnameOrIPOrDie($opt_hostname);
    goodNFSExportOrDie($opt_export); 
    goodNFSOptionsOrDie($opt_options);

    editFilesystem($opt_edit, $opt_hostname, $opt_export, $opt_options);
    if (defined($opt_unmount))
    {
	exit(2) if (!unmountFilesystem($opt_mount));
    }
    if (defined($opt_mount))
    {
	exit(2) if (!mountFilesystem($opt_edit));
    }
}
elsif (defined($opt_mount))
{
    exit(3) if (!mountFilesystem($opt_mount));
}
elsif (defined($opt_unmount))
{
    exit(3) if (!unmountFilesystem($opt_unmount));
}

exit(0);

#
# Subroutine: addNFSFilesystem
#
# Adds an NFS Filesystem to fstab
#
sub addNFSFilesystem()
{
    my @dirs = getNetDirs();
    my %hash;
    
    # convert to hash table
    foreach my $dir (@dirs) 
    {
	$hash{$dir} = $dir;
    }

    # find first available /net/homex entry
    my $i;
    for ($i = 1; $hash{"home".$i}; $i++){};
    my $mount_point = "/net/home".$i;
    
    # Create new mount point
    if (!mkpath($mount_point, 0, 0755))
    {
	trace("Unable to create ".$mount_point);
	error(MSG_COULD_NOT_CREATE_DIRECTORY, $mount_point);
	exit(3);
    }
    
    # Append new filesystem to /etc/fstab
    eval('openOrDie(FSTAB, ">> /etc/fstab");');
    eval('printOrDie(FSTAB, "$opt_hostname:$opt_export\t$mount_point\tnfs\t$opt_options\t0\t0\n");') if (!$@);
    if ($@)   # Unable to update /etc/fstab, remove mount point
    {
	if (!rmdir($mount_point))
	{
	    trace("Could not rmdir $mount_point");
	    error(MSG_COULD_NOT_DELETE_DIR, $mount_point);
	}
	
	exit(3);
    }

    closeOrDie(FSTAB);

    audit(MSG_NFS_ADD, "$opt_hostname:$opt_export", $mount_point, $opt_options);

    return $mount_point;
}

#
# subroutine: editFilesystem
#
# Edits the specified filesystem (ie. replace current values in /etc/fstab)
#
sub editFilesystem()
{
    my ($mount_point, $hostname, $export, $options) = @_;

    openOrDie(FSTAB, "/etc/fstab");
    openOrDie(NEWFSTAB, "> /etc/fstab.new");
    while (<FSTAB>)
    {
	if (/^\#/ || /^\s*$/)
	{
	    printOrDie(NEWFSTAB,$_);
	    next;
	}

	($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	if ($fs_file eq $mount_point)
        {
	    printOrDie(NEWFSTAB, "$hostname:$export\t$mount_point\tnfs\t$options\t0\t0\n");
	}  
	else
	{
	    printOrDie(NEWFSTAB, $_);
	}
    }
    closeOrDie(NEWFSTAB);
    closeOrDie(FSTAB);

    # Save backup copy of /etc/fstab to /etc/fstab.bk
    copy("/etc/fstab", "etc/fstab.bk");
    if (!rename("/etc/fstab.new", "/etc/fstab"))
    {
	trace("Unable to rename /etc/fstab.new to /etc/fstab");
	error(MSG_COULD_NOT_RENAME_FILE, "/etc/fstab.new", "/etc/fstab");
    }

    audit(MSG_NFS_EDIT, $mount_point, "$hostname:$export", $options);

    return 1;
}

#
# subroutine: mountFilesystem
#
# Mounts the filesystem specified at mount point in /etc/fstab
#
sub mountFilesystem()
{
    my ($mount_point) = @_;

    if (system("mount", $mount_point))
    {
	trace "Unable to mount $mount_point";
	error(MSG_UNABLE_TO_MOUNT, $mount_point);
	return(0);
    }

    audit(MSG_NFS_MOUNT, $mount_point);
    
    return(1);
}

#
# Subroutine: unmountFilesystem
#
# Unmounts the filesystem at the specified mount point
#
sub unmountFilesystem()
{
    my ($mount_point) = @_;


    if (system("umount", $mount_point))
    {
	trace "Unable to unmount $mount_point";
	error(MSG_UNABLE_TO_UNMOUNT, $mount_point);
	return(0);
    }

    audit(MSG_NFS_UNMOUNT, $mount_point);

    return(1);
}

#
# Subroutine: deleteFilesystem
#
# Removes the specified filesystem (specified by mount point) from /etc/fstab and deletes the mount point
#
sub deleteFilesystem()
{
    my ($mount_point) = @_;

    openOrDie(FSTAB, "/etc/fstab");
    openOrDie(NEWFSTAB, "> /etc/fstab.new");
    while (<FSTAB>)
    {
	if (/^\#/ || /^\s*$/)
	{
	    printOrDie(NEWFSTAB, $_);
	    next;
	}

	($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	next if ($fs_file eq $mount_point);  
	printOrDie(NEWFSTAB, $_);
    }
    closeOrDie(NEWFSTAB);
    closeOrDie(FSTAB);

    # Save backup copy of /etc/fstab to /etc/fstab.bk
    copy("/etc/fstab", "etc/fstab.bk");
    if (!rename("/etc/fstab.new", "/etc/fstab"))
    {
	trace("Unable to rename /etc/fstab.new to /etc/fstab");
	error(MSG_COULD_NOT_RENAME_FILE, "/etc/fstab.new", "/etc/fstab");
    }

    audit(MSG_NFS_DELETE, $mount_point);
    return 1;
}

#
# Subroutine: printNFSFilesystems
#
# Prints the list of NFS filesystems
#
sub printNFSFilesystems() {
    my %mounts = getProcMounts();
    my ($entry, @filesystems);

    foreach $entry (getFstab())
    {
	# Ignore non-nfs entries
	next if (${$entry}[2] ne "nfs");
        # Ignore extra entries if searching for a specific mount point
        next if ($opt_list ne "" && 
		 $opt_list ne ${$entry}[1]);

        push @filesystems,  join " ", ${$entry}[0], 
                                      ${$entry}[1], 
                                      ${$entry}[3],
                                      ($mounts{${$entry}[1]}? "active" : "inactive");
    }

    print join("\n", @filesystems) . "\n";
}



#
# Subroutine: getFstab()
#
# Returns an array representation of fstab
#
sub getFstab() {
    return getFilesystemTable("$fstab_file");
}

#
# Subroutine: getProcMounts()
#
# Returns a hash table (based on mountpoint) representation of /proc/mounts
#
sub getProcMounts() {
    my %mounts;

    foreach $mount (getFilesystemTable("/proc/mounts"))
    {
	$mounts{$mount->[1]} = $mount;
    }

    return %mounts;
}

#
# Subroutine: getFilesystemTable
#
# Returns an array representation of filesytem tables
#
sub getFilesystemTable($) {
    my ($table) = @_;

    my @file_systems;

    open FSTAB, "< $table";
    while (<FSTAB>)
    {
	next if (/^\#/ || /^\s*$/);
	($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	next if !defined($fs_spec);

	push @file_systems, [$fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno];
    }
    close FSTAB;

    return @file_systems;
}


#
#
#
#

sub getNetDirs()
{
    if (!opendir(NETDIR, "/net"))
    {
	trace("Unable to open /net directory");
	error(MSG_COULD_NOT_OPEN_DIRECTORY, "/net");
	exit(4);
    }
    my @files = readdir NETDIR;
    closedir NETDIR;
    
    return @files;
}




sub goodNFSExportOrDie()
{
    my ($export) = @_;

    if ($export =~ /\s+/)                        # includes whitespace
    {
	error(MSG_NFS_BAD_EXPORT_FORMAT, $export);
	exit(3);
    }
    
    return(1);
}


sub goodNFSOptionsOrDie()
{
    my ($options) = @_;

    if ($options =~ /\s+/)                        # includes whitespace
    {
	error(MSG_NFS_BAD_OPTIONS_FORMAT, $options);
	exit(3);
    }
    
    return(1);
}
