#!/usr/bin/perl
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

use strict;
BEGIN { $^W = 1; }

my ($rc, $stdout, $stderr, @opts, $setlocaleHTTP_usage, $httpd_locale);
my ($ihs_root, $ihs_adminfile, $ihs_conffile, $ihs_keyfile, $curr_date, $tempfile1, $tempfile2, $adminsav, $ihssav);

use ASM::Common;
use ASM::Exec;
use ASM::Log;

setFacility("initsys");

use vars qw/ $opt_help $opt_locale /;

$ihs_root = "/opt/IBMHTTPServer";
$ihs_adminfile = $ihs_root . "/conf/admin.conf";
$ihs_conffile = $ihs_root . "/conf/httpd.conf";
$ihs_keyfile = "/usr/bin/ikeyman"; 

my $csm_root = "/opt/CSM/WebServer";
my $csm_conffile = $csm_root . "/conf/httpd.conf";

# Parse the arguments
@opts = ("locale=s");
$setlocaleHTTP_usage = "\t\tor\nsetlocaleHTTP.pl [ --locale=<name> ] [ --help ]   where the default locale is \"en_US\"";

commonInit(["locale=s", "help"], $setlocaleHTTP_usage);

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
   trace("$exitmsg");
   trace("exit $exitrc");
   trace("====== ".`date|tr -d "\n"`." ======");
   if ($exitrc != 0)
   {
      error(MSG_SETLOCALE_HTTP_FAILED);
   }
   else
   {
      audit(MSG_SETLOCALE_HTTP_COMPLETED);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

#
#  Map the given locale to the ones IHS uses in
#  its config files.
#
sub map_locale
{
   my ($curr_locale) = @_;
   my ($new_locale);

   chomp($curr_locale);
   $new_locale = lc($curr_locale);
   $_ = $new_locale;
   if (/^ja_/)
   {
      $new_locale = "Ja_JP";
   }
   elsif (/^zh_tw/)
   {
      $new_locale = "Zh_TW";
   }
   elsif (/^zh_/)
   {
      $new_locale = "Zh_CN";
   }
   elsif (/^ko_/)
   {
      $new_locale = "ko_KR";
   }
   elsif (/^de_/)
   {
      $new_locale = "de_DE";
   }
   elsif (/^fr_/)
   {
      $new_locale = "fr_FR";
   }
   elsif (/^it_/)
   {
      $new_locale = "it_IT";
   }
   elsif (/^pt_/)
   {
      $new_locale = "pt_BR";
   }
   elsif (/^es_/)
   {
      $new_locale = "es_ES";
   }
   elsif (/^en_/)
   {
      $new_locale = "en_US";
   }
   else
   {
      #  Unsupported locale.
      $new_locale = "";
      trace("Unsupported IHS locale $curr_locale detected.");
   }
   
   return($new_locale);
}

&setup_signals(); 

# Get command line options and check 'em out to see if they're okay.
 
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   &cleanup_exit("Error:  Unknown problem with parsing arguments.", 2);
}

if($opt_help)
{
   &printUsage; 
}

if ($opt_locale)
{
   $httpd_locale = &map_locale($opt_locale);
   if ($httpd_locale eq "")
   {
      #  Unknown/Unsupported locale.
      &cleanup_exit("IHS locale mapping has been aborted.", 1);
   }
}
else
{
   #  Default to English
   $opt_locale = "en_US";
   $httpd_locale = "en_US";
}

if (-f $csm_conffile)
{
   chomp($curr_date = `date -u +"%T_%j"`);
   $tempfile1 = "/tmp/tempcsm." . $curr_date;
   my $csmsav = "$csm_conffile.$curr_date";

   local(*IN, *OUT);
   openOrDie(IN, "< $csm_conffile");
   openOrDie(OUT, "> $tempfile1");
   while(<IN>)
   {
     if (m!^DirectoryIndex IBMAppliance!) {
       $_ = "DirectoryIndex IBMAppliance/html/$opt_locale/index.html index.html index.htm\n";
     }
     print OUT;
   }
   close IN;
   close OUT;
   $rc = runit(["cp", "-f", $csm_conffile, $csmsav], \$stdout, \$stderr);
   if ($rc == 0) {
     $rc = runit(["cp", "-f", $tempfile1, $csm_conffile], \$stdout, \$stderr);
   }
   if ($rc == 0) {
     $rc = runit(["rm", "-f",  $tempfile1], \$stdout, \$stderr);
   }
}

if (-f $ihs_adminfile)
{
   chomp($curr_date = `date -u +"%T_%j"`);
   $tempfile1 = "/tmp/tempadmin." . $curr_date;
   $adminsav = "$ihs_adminfile.$curr_date";

   openOrDie(ADMIN, "< $ihs_adminfile");
   openOrDie(NEWADMIN, "> $tempfile1");
   while(<ADMIN>)
   {
      if (/^Alias.*$/)
      {
         s/[a-zZJ][a-z]_[A-Z][A-Z]/$httpd_locale/;
      }
      if (/^Include.*$/)
      {
         s/\.[a-zZJ][a-z]_[A-Z][A-Z]/\.$httpd_locale/;
      }
      if (/^DocumentRoot.*$/)
      {
         s/[a-zZJ][a-z]_[A-Z][A-Z]/$httpd_locale/;
      }
      print NEWADMIN;
   }
   close NEWADMIN;
   close ADMIN;
   $rc = runit(["cp", "-f", $ihs_adminfile, $adminsav], \$stdout, \$stderr);
   if ($rc == 0) {
     $rc = runit(["cp", "-f", $tempfile1, $ihs_adminfile], \$stdout, \$stderr);
   }
   if ($rc == 0) {
     $rc = runit(["rm", "-f",  $tempfile1], \$stdout, \$stderr);
   }
 
}

#  IHS production server will no longer have default Locale directories
#  ASM will create a set of default web pages in one directory.
#if (-f $ihs_conffile)
#{
#   chomp($curr_date = `date -u +"%T_%j"`);
#   $tempfile1 = "/tmp/temphttpd." . $curr_date;
#   $ihssav = $ihs_conffile . "." . $curr_date;
#
#   openOrDie(HTTPD, "< $ihs_conffile");
#   openOrDie(NEWHTTPD, "> $tempfile1");
#   while(<HTTPD>)
#   {
#      if (/^DocumentRoot.*$/)
#      {
#         s/[a-zZJ][a-z]_[A-Z][A-Z]/$httpd_locale/;
#      }
#      if (/^<Directory.*$/)
#      {
#         s/[a-zZJ][a-z]_[A-Z][A-Z]/$httpd_locale/;
#      }
#      print NEWHTTPD;
#   }
#   close NEWHTTPD;
#   close HTTPD;
#   $rc = runit(["cp", "-f", $ihs_conffile, $ihssav], \$stdout, \$stderr);
#   $rc = runit(["cp", "-f", $tempfile1, $ihs_conffile], \$stdout, \$stderr);
#   $rc = runit(["rm", "-f",  $tempfile1], \$stdout, \$stderr);
# 
#}

if (-f $ihs_keyfile)
{
   chomp($curr_date = `date -u +"%T_%j"`);
   $tempfile1 = "/tmp/tempkey." . $curr_date;

   if (!open(KEYF, "< $ihs_keyfile"))
   {
      trace("Couldn't open $ihs_keyfile.");
      trace("Continuing since its not a required file.");
   }
   else
   {
      open(NEWKEYF, "> $tempfile1");
      while(<KEYF>)
      {
         s/^export IKMLANG=.*/export IKMLANG='$httpd_locale'/;
         print NEWKEYF;
      }
      close NEWKEYF;
      close KEYF;
      $rc = runit(["cp", "-f", $tempfile1, $ihs_keyfile], \$stdout, \$stderr);
      $rc = runit(["rm", "-f",  $tempfile1], \$stdout, \$stderr);
   }

&cleanup_exit("", 0);
}
