#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# uninstallImage.pl - uninstall a image
#    uninstallImage --pkg_name=<Package Name>
#
# Wayne Gao, <waynegao@us.ibm.com> (5 November 1999)
#

use ASM::Common;
use Getopt::Long;

use strict;
my ($eSP_BIN, $rc, $stdout, $stderr, $uninstallImage_usage, @opts);
my ($pkg_name, $ug_type, $removed, $modified, $added, $rpm, $failure, $fn, $status);

$eSP_BIN = "/opt/CSM/scripts";

require "$eSP_BIN/common_install.pl";

$uninstallImage_usage = "\t\tor\nuninstallImage.pl --pkg_name=<Upgrade Package Name> ";
commonInit(["pkg_name=s"],"");

setFacility('upgrade');

use vars qw/ $opt_help $opt_pkg_name $IMG_DIR $BK_PKG $BK_DIR $BK_IMG $setup_data_file /;

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;

   trace("$exitmsg");
   trace("exit $exitrc");
   if ($exitrc != 0)
   {
      error(MSG_UNINSTALL_FAILED);
   }
   else
   {
      audit(MSG_UNINSTALL_COMPLETED);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

# Set "upgradeFailed=1" in csm.data
sub set_failure_flag {
  local *IN;
  local *OUT;
  if (!open(IN, "< $setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);
    return;
  }
  my @lines = <IN>;
  close(IN);
  my $found = 0;
  foreach my $line (@lines) {
    if ($line =~ /upgradeFailed/ && !$found) {
      $line = "upgradeFailed=1\n";
      $found = 1;
    }
  }
  if (!$found) {
    push(@lines, "upgradeFailed=1\n");
  }
  if (!open(OUT, ">$setup_data_file")) {
    error(MSG_COULD_NOT_OPEN_FILE, $setup_data_file);;
    return;
  }
  print OUT @lines;
  close(OUT);
}

&setup_signals();

@opts = ("admin=s", "pkg_name=s");
&GetOptions(@opts);

$pkg_name = $opt_pkg_name;

if ($pkg_name eq "")
{
   error(MSG_USAGE_ERROR, "uninstallImage.pl", $uninstallImage_usage);
   &cleanup_exit("Error: Required parameter pkg_name not specified!", 2);
}

$BK_PKG = "$BK_DIR/$pkg_name";
if (! -e $BK_PKG)
{
   error(MSG_PACKAGE_NOT_INSTALLED, $pkg_name);
   &cleanup_exit("Package $pkg_name is not installed. Exit.", 4);
}

trace("Package Name= $pkg_name");
$BK_IMG="$BK_PKG/bk_image";

# Extract information from pkgInfo.txt file
if (-e "$BK_PKG/pkgInfo.txt")
{
   trace("++++ Extract information from pkgInfo.txt file +++++");
   chop($ug_type = `grep "UgType" $IMG_DIR/pkgInfo.txt|grep -v "\#"|cut -f2 -d "="|tr -d " "`);
}
else
{
   trace("ERROR: File $BK_IMG/pkgInfo.txt is missing.");
   trace("Continuing uninstall....");
}

$removed="$BK_PKG/removed.lst";
$modified="$BK_PKG/modified.lst";
$added="$BK_PKG/added.lst";
$rpm="$BK_PKG/rpm.lst";
openOrDie(INPT_rm,$removed);
openOrDie(INPT_ad,$added);
openOrDie(INPT_rpm,$rpm);


$failure=0;
trace("RPMS applied with the software update will not be removed using the rpm command.");
trace("Old files will overlay the files in the software update.");
# Remove all rpm packages
if (0) {
  # NO, don't do this
  # since all we're doing to restore is copy the files back, we can get
  # into trouble if we 'rpm -e' which will run uninstall scripts, but we
  # never install another rpm so no install scripts get run.
  # Better to just copy the files back over.
  trace("++++ Remove rpm packages ++++");
  while (<INPT_rpm>)
    {
      chop($_);
      trace("++ found $_");
      trace("rpm --nodeps -e $_");
      $rc = runit(["rpm --nodeps -e $_"], \$stdout, \$stderr);
      if ($rc != 0)
	{
	  trace("Failed to remove $_");
	  $failure = 1;
	}
    }
}

# Remove added files  (rpm -e should have done the rpm added files)
trace("++++ Remove added files ++++");
while (<INPT_ad>)
{
   chop($_);
   if (/#/) {
      trace("++ found $_");
      $fn=`echo $_|cut -f1 -d "#"`;
      trace("rm -rf $fn");
      $rc = runit(["rm -rf $fn"], \$stdout, \$stderr);
      if ($rc != 0)
      {
         $failure=1;
         trace("Failed to remove $fn");
      }
   }
}

#
#  Simplify the recovery process by using tar to recover files
#  Also, use a recover.sh script in the update file to
#  handle the specialized recovery such as kernel updates,
#  rpm DB manipulation, WebSphere recovery, etc...


$rc = runit(["uncompress < $BK_IMG/backup.tar.Z|tar --same-owner -xpPf - "], \$stdout, \$stderr);
if ($rc != 0)
{
   error(MSG_UNABLE_TO_RECOVER_FILES, "$BK_IMG/backup.tar.Z");
   trace("Error:  Recovery of files failed from backup image: $BK_IMG/backup.tar.Z tar rc = $rc");
   &cleanup_exit("Error: Recover of files failed!.", 86);
}

close INPT_rm;
close INPT_ad;
close INPT_rpm;

# For recover kernel upgrade, run lilo to update the map
if (($ug_type eq "kernel_only") || ($ug_type eq "kernel_app"))
{
   trace("++ /sbin/lilo");
   $rc = runit(["/sbin/lilo"], \$stdout, \$stderr);
   if ( $rc != 0 )
   {
      trace("ERROR: failed to run lilo.");
      trace("Failed to update boot map.");
      error(MSG_UNABLE_TO_RUN_LILO);
      error(MSG_BOOT_IMAGE_UPDATE_FAILED);
      $failure=1;
   }
   else
   {
      error(MSG_BOOT_IMAGE_UPDATE_COMPLETED);
      trace("Successfully updated boot map");
   }
}

# call recovery program to complete recovery of the old image
trace("++++ Call recovery program to complete the uninstall of the upgrade ++++");
trace("$BK_PKG/recover.sh");
$status = runit(["$BK_PKG/recover.sh"], \$stdout, \$stderr);
trace("$BK_PKG/recover.sh exit $status");

#  Set the failure flag in csm.data for the ASM GUI failure message
&set_failure_flag();

# Check if the un-install is successful. Remove the backup 
# package. if the un-install is successful.
if ($status == 0)
{
   trace("++++ Un-Install successful. Remove the backup ++++");
   trace("rm -rf $BK_PKG");
   $rc = runit(["rm -rf $BK_PKG"], \$stdout, \$stderr);
   trace("$pkg_name upgrade \"Removal of software upgrade completed (uninstall).\"");
   &cleanup_exit("Image uninstalled Successfully", 0);
}
else
{
  trace("$pkg_name upgrade \"Removal of software upgrade failed (uninstall).\"");
  trace("Software Upgrade Package may be incomplete.");
  &cleanup_exit("ERROR: ++++ Some failure on un-installation.", 88);
}
