#
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

=head1 NAME

ASM::Binhash - tools for storing a hash with arbitrarily encoded strings

=head1 SYNOPSIS

use ASM::BinHash;

my %hash;

$hash{KEY1} = "Message1";
$hash{KEY2} = "Message2";

writeHash("/path/filename", \%hash, "ASCII");

...

my ($href, $encoding) = readHash("/path/filename");

print "Message KEY1 is " . $href->{KEY1} . "\n";
print "Message KEY2 is " . $href->{KEY2} . "\n";


=head1 DESCRIPTION

The B<ASM::BinHash> module allows storing and retrieving hashes
with translated strings into files without messing up the encoding.

=cut

package ASM::BinHash;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(writeHash readHash);




sub writeHash {
  my ($filename, $hash, $encoding) = @_;

  local(*OUT);
  if (open(OUT, ">$filename")) {
    
    writeString(\*OUT, $encoding);
    foreach my $key (keys %$hash) {
      writeString(\*OUT, $key);
      writeString(\*OUT, $hash->{$key});
    }
    close(OUT);
  } else {
    die "$filename: $!";
  }
  1;
}

# Returns (\%hash, $encodingName)
sub readHash {
  my ($filename) = @_;

  my %msg;

  local(*IN);

  my $ENCODING;
  
  if (open(IN, "<$filename")) {
    
    $ENCODING = readString(\*IN);
    
    while (!eof(\*IN)) {
      # Another message
      my ($key, $value);
      eval { $key = readString(\*IN) };
      die "$@ while reading key in $filename" if $@;
      eval { $value = readString(\*IN) };
      die "$@ while reading value in $filename" if $@;
      $msg{$key} = $value;
    }
    close(IN);
  }
  return (\%msg, $ENCODING);
}


sub readString {
  my ($handle) = @_;

  my ($len,$bytes,$sum);
  my ($buf);
  my ($result);

  $len = readInteger($handle);
  if ($len != ($result = read($handle, $bytes, $len))) {
    die "Tried to read $len bytes, only got $result: $!";
  }
  $sum = readInteger($handle);
  #print "Length is $len... string is $bytes... sum says $sum\n";
  my $computed = checksum($bytes);
  if ($sum != $computed) {
    die "bad checksum computed $computed != file $sum";
  }
  return $bytes;
}

sub readInteger {
  my ($handle) = @_;
  my ($buf, $result);
  if (2 != ($result = read($handle, $buf, 2))) {
    die "Tried to read 2 bytes, only got $result: $!";
  }
  return unpackInteger($buf);
}

sub unpackInteger {
  my ($buf) = @_;

  return scalar unpack('n', $buf);
}

sub checksum {
  my ($bytes) = @_;

  my (@bytes) = unpack('C*', $bytes); # unpack to a list of unsigned numbers
  my ($sum) = 0;
  for (my $i = 0; $i < length($bytes); $i++) {
    #print "sum starts $sum, byte is $bytes[$i], result is ";
    $sum = ($sum + $bytes[$i]) & 0xffff;
    #print "$sum\n";
  }

  return $sum;
}


# Write a complete string according to the format defined above
sub writeString {
  my ($handle,$string) = @_;

  writeInteger($handle, length($string));
  writeBuffer($handle, $string);
  writeInteger($handle, checksum($string));
}

# Just write the bytes in the given string as-is
sub writeBuffer {
  my ($handle, $value) = @_;
  my $len = length($value);
  my $result;
  if ($len != ($result = syswrite($handle, $value, $len))) {
    die "Tried to write $len bytes, only wrote $result: $!";
  }
}

sub writeInteger {
  my ($handle,$value) = @_;
  my $buf = packInteger($value);
  my $result;
  if (2 != ($result = syswrite($handle, $buf, 2))) {
    die "Tried to write 2 bytes, only wrote $result: $!";
  }
}

sub packInteger {
  my ($i) = @_;
  return pack('n', $i);
}


1;

